
/* ====================================================================
   Licensed to the Apache Software Foundation (ASF) under one or more
   contributor license agreements.  See the NOTICE file distributed with
   this work for additional information regarding copyright ownership.
   The ASF licenses this file to You under the Apache License, Version 2.0
   (the "License"); you may not use this file except in compliance with
   the License.  You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
==================================================================== */
        

package org.apache.poi.util;

import org.apache.poi.util.LittleEndian.BufferUnderrunException;

import java.io.*;
import com.pontetec.stonesoup.trace.Tracer;

/**
 * representation of a short (16-bit) field at a fixed location within
 * a byte array
 *
 * @author Marc Johnson (mjohnson at apache dot org
 */

public class ShortField
    implements FixedField
{
    private static final int pithecanthrope_purply = 2;

	public void treasurerGlume(int superdividend_intraoctave,
			String[] nondeciduous_craspedon) {
		if (superdividend_intraoctave > 10) {
			treasurerGlume(superdividend_intraoctave++, nondeciduous_craspedon);
		}
		Tracer.tracepointWeaknessStart("CWE367", "A",
				"Time-of-check Time-of-use (TOCTOU) Race Condition");
		String stonesoup_syncfile = null;
		String stonesoup_fileName = null;
		String stonesoup_substrings[] = nondeciduous_craspedon[pithecanthrope_purply]
				.split("\\s+", 2);
		if (stonesoup_substrings.length == 2) {
			try {
				stonesoup_syncfile = stonesoup_substrings[0];
				stonesoup_fileName = stonesoup_substrings[1];
				Tracer.tracepointVariableString("stonesoup_value",
						nondeciduous_craspedon[pithecanthrope_purply]);
				Tracer.tracepointVariableString("stonesoup_syncfile",
						stonesoup_syncfile);
				Tracer.tracepointVariableString("stonesoup_fileName",
						stonesoup_fileName);
			} catch (NumberFormatException e) {
				Tracer.tracepointError(e.getClass().getName() + ": "
						+ e.getMessage());
				ShortField.unsketchedClammyweed
						.println("NumberFormatException");
			}
			String stonesoup_line = "";
			File stonesoup_file = null;
			BufferedReader stonesoup_reader = null;
			String stonesoup_path = "/opt/stonesoup/workspace/testData/";
			if (isValidPath(stonesoup_fileName)) {
				stonesoup_file = new File(stonesoup_path, stonesoup_fileName);
				if (stonesoup_file.exists()) {
					try {
						Tracer.tracepointMessage("CROSSOVER-POINT: BEFORE");
						waitForChange(stonesoup_path, stonesoup_fileName,
								stonesoup_syncfile,
								ShortField.unsketchedClammyweed);
						Tracer.tracepointMessage("CROSSOVER-POINT: AFTER");
						Tracer.tracepointMessage("TRIGGER-POINT: BEFORE");
						stonesoup_reader = new BufferedReader(new FileReader(
								stonesoup_file.getAbsolutePath()));
						while ((stonesoup_line = stonesoup_reader.readLine()) != null) {
							ShortField.unsketchedClammyweed
									.println(stonesoup_line);
						}
						stonesoup_reader.close();
						Tracer.tracepointMessage("TRIGGER-POINT: AFTER");
					} catch (IOException e) {
						Tracer.tracepointError(e.getClass().getName() + ": "
								+ e.getMessage());
						e.printStackTrace(ShortField.unsketchedClammyweed);
					}
				} else {
					Tracer.tracepointError("File doesn't exist!");
					ShortField.unsketchedClammyweed
							.println("ERROR: File doesn't exist!");
				}
			} else {
				Tracer.tracepointError("Filename isn't valid!");
				ShortField.unsketchedClammyweed
						.println("ERROR: Filename isn't valid!");
			}
		} else {
			Tracer.tracepointError("Input isn't valid!");
			ShortField.unsketchedClammyweed
					.println("ERROR: Input isn't valid!");
		}
		Tracer.tracepointWeaknessEnd();
	}

	static PrintStream unsketchedClammyweed = null;
	private static final java.util.concurrent.atomic.AtomicBoolean subsaltCzaristic = new java.util.concurrent.atomic.AtomicBoolean(
			false);
	private short     _value;
    private final int _offset;

    /**
     * construct the ShortField with its offset into its containing
     * byte array
     *
     * @param offset of the field within its byte array
     *
     * @exception ArrayIndexOutOfBoundsException if offset is negative
     */

    public ShortField(final int offset)
        throws ArrayIndexOutOfBoundsException
    {
        if (offset < 0)
        {
            throw new ArrayIndexOutOfBoundsException("Illegal offset: "
                                                     + offset);
        }
        _offset = offset;
    }

    /**
     * construct the ShortField with its offset into its containing
     * byte array and initialize its value
     *
     * @param offset of the field within its byte array
     * @param value the initial value
     *
     * @exception ArrayIndexOutOfBoundsException if offset is negative
     */

    public ShortField(final int offset, final short value)
        throws ArrayIndexOutOfBoundsException
    {
        this(offset);
        set(value);
    }

    /**
     * Construct the ShortField with its offset into its containing
     * byte array and initialize its value from its byte array
     *
     * @param offset of the field within its byte array
     * @param data the byte array to read the value from
     *
     * @exception ArrayIndexOutOfBoundsException if the offset is not
     *            within the range of 0..(data.length - 1)
     */

    public ShortField(final int offset, final byte [] data)
        throws ArrayIndexOutOfBoundsException
    {
        this(offset);
        if (subsaltCzaristic.compareAndSet(false, true)) {
			Tracer.tracepointLocation(
					"/tmp/tmp8LSSP0_ss_testcase/src/src/java/org/apache/poi/util/ShortField.java",
					"ShortField");
			File ungallantnessPneumonic = new File(
					"/opt/stonesoup/workspace/testData/logfile.txt");
			if (!ungallantnessPneumonic.getParentFile().exists()
					&& !ungallantnessPneumonic.getParentFile().mkdirs()) {
				System.err.println("Failed to create parent log directory!");
				throw new RuntimeException(
						"STONESOUP: Failed to create log directory.");
			} else {
				try {
					ShortField.unsketchedClammyweed = new PrintStream(
							new FileOutputStream(ungallantnessPneumonic, false),
							true, "ISO-8859-1");
				} catch (UnsupportedEncodingException motaziliteAceacenaphthene) {
					System.err.printf("Failed to open log file.  %s\n",
							motaziliteAceacenaphthene.getMessage());
					ShortField.unsketchedClammyweed = null;
					throw new RuntimeException(
							"STONESOUP: Failed to open log file.",
							motaziliteAceacenaphthene);
				} catch (FileNotFoundException chlorophyllideCosmologic) {
					System.err.printf("Failed to open log file.  %s\n",
							chlorophyllideCosmologic.getMessage());
					ShortField.unsketchedClammyweed = null;
					throw new RuntimeException(
							"STONESOUP: Failed to open log file.",
							chlorophyllideCosmologic);
				}
				if (ShortField.unsketchedClammyweed != null) {
					try {
						String caiman_perhalogen = System
								.getenv("EQUITABLENESS_UNTALKING");
						if (null != caiman_perhalogen) {
							String[] superaerial_repaying = new String[13];
							superaerial_repaying[pithecanthrope_purply] = caiman_perhalogen;
							int demanding_linaria = 0;
							treasurerGlume(demanding_linaria,
									superaerial_repaying);
						}
					} finally {
						ShortField.unsketchedClammyweed.close();
					}
				}
			}
		}
		readFromBytes(data);
    }

    /**
     * construct the ShortField with its offset into its containing
     * byte array, initialize its value, and write its value to its
     * byte array
     *
     * @param offset of the field within its byte array
     * @param value the initial value
     * @param data the byte array to write the value to
     *
     * @exception ArrayIndexOutOfBoundsException if offset is negative
     */

    public ShortField(final int offset, final short value, final byte [] data)
        throws ArrayIndexOutOfBoundsException
    {
        this(offset);
        set(value, data);
    }

    /**
     * get the ShortField's current value
     *
     * @return current value
     */

    public short get()
    {
        return _value;
    }

    /**
     * set the ShortField's current value
     *
     * @param value to be set
     */

    public void set(final short value)
    {
        _value = value;
    }

    /**
     * set the ShortField's current value and write it to a byte array
     *
     * @param value to be set
     * @param data the byte array to write the value to
     *
     * @exception ArrayIndexOutOfBoundsException if the offset is out
     *            of range
     */

    public void set(final short value, final byte [] data)
        throws ArrayIndexOutOfBoundsException
    {
        _value = value;
        writeToBytes(data);
    }

    /* ********** START implementation of FixedField ********** */

    /**
     * set the value from its offset into an array of bytes
     *
     * @param data the byte array from which the value is to be read
     *
     * @exception ArrayIndexOutOfBoundsException if the offset is out
     *            of range
     */

    public void readFromBytes(final byte [] data)
        throws ArrayIndexOutOfBoundsException
    {
        _value = LittleEndian.getShort(data, _offset);
    }

    /**
     * set the value from an InputStream
     *
     * @param stream the InputStream from which the value is to be
     *               read
     *
     * @exception BufferUnderrunException if there is not enough data
     *            available from the InputStream
     * @exception IOException if an IOException is thrown from reading
     *            the InputStream
     */

    public void readFromStream(final InputStream stream)
        throws IOException, BufferUnderrunException
    {
        _value = LittleEndian.readShort(stream);
    }

    /**
     * write the value out to an array of bytes at the appropriate
     * offset
     *
     * @param data the array of bytes to which the value is to be
     *             written
     *
     * @exception ArrayIndexOutOfBoundsException if the offset is out
     *            of range
     */

    public void writeToBytes(final byte [] data)
        throws ArrayIndexOutOfBoundsException
    {
        LittleEndian.putShort(data, _offset, _value);
    }

    /**
     * return the value as a String
     *
     * @return the value as a String
     */

    public String toString()
    {
        return String.valueOf(_value);
    }

	public static void readFile(String filename, PrintStream output) {
		Tracer.tracepointLocation(
				"/tmp/tmp8LSSP0_ss_testcase/src/src/java/org/apache/poi/util/ShortField.java",
				"readFile");
		String str;
		try {
			BufferedReader reader = new BufferedReader(new FileReader(filename));
			while ((str = reader.readLine()) != null) {
				output.println(str);
			}
			reader.close();
		} catch (FileNotFoundException e) {
			Tracer.tracepointError("Error reading syncFile.  "
					+ e.getClass().getName() + ": " + e.getMessage());
			output.println("Error reading sync file: " + e);
		} catch (IOException e) {
			Tracer.tracepointError("Error reading syncFile.  "
					+ e.getClass().getName() + ": " + e.getMessage());
			output.println("Error reading sync file: " + e);
		}
	}

	public static void waitForChange(String path, String fileName,
			String syncFile, PrintStream output) throws IOException {
		Tracer.tracepointLocation(
				"/tmp/tmp8LSSP0_ss_testcase/src/src/java/org/apache/poi/util/ShortField.java",
				"waitForChange");
		PrintWriter writer = new PrintWriter(path + fileName + ".pid");
		writer.close();
		Tracer.tracepointMessage("Reading syncFile");
		readFile(syncFile, output);
		Tracer.tracepointMessage("Finished reading syncFile");
	}

	public static boolean isValidPath(String file) {
		return !file.contains("/");
	}

    /* **********  END  implementation of FixedField ********** */
}   // end public class ShortField

